package com.redstrate.watersymbol.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.maps.tiled.TiledMap;
import com.badlogic.gdx.maps.tiled.TiledMapRenderer;
import com.badlogic.gdx.maps.tiled.TmxMapLoader;
import com.badlogic.gdx.maps.tiled.renderers.OrthogonalTiledMapRenderer;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.utils.ScreenUtils;
import com.redstrate.watersymbol.*;
import com.redstrate.watersymbol.maps.TestMap;
import org.w3c.dom.Text;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class GameScreen implements Screen {
    private final WaterSymbol game;
    private boolean hasStartedMoving = false;
    TiledMap tiledMap;
    OrthographicCamera camera;
    TiledMapRenderer tiledMapRenderer;

    public Map map;
    public ArrayList<Unit> units = new ArrayList<>();
    Unit.Team currentTurnTeam = Unit.Team.Player;
    Unit currentUnit = null;
    int currentUnitIndex = 0;
    int currentTurn = 0;

    Texture playerTexture;
    Sprite playerSprite;

    Animation<TextureRegion> playerIdleAnimation;

    Animation<TextureRegion> enemyIdleAnimation;

    ArrowSpritesheet arrowSpritesheet;

    // data for in-turn actions
    boolean currentlyTakingAction = false;
    List<Vector2> currentMovementList;

    boolean currentlyAwaitingAuxilaryAction = false;

    int cursorX = 0, cursorY = 0;
    Texture cursorTexture;

    private Stage stage;
    private Table table;

    TextButton unitNameButton;

    boolean currentlyParticipatingInStageEvent = false;
    Texture stageTexture;

    GameScreen(WaterSymbol game) {
        this.game = game;

        camera = new OrthographicCamera();
        camera.setToOrtho(false, 800, 480);
        camera.zoom = 2.0f;

        map = new TestMap();
        tiledMap = new TmxMapLoader().load(map.getMapFilename());
        map.setup(this);

        tiledMapRenderer = new OrthogonalTiledMapRenderer(tiledMap);

        playerTexture = new Texture(Gdx.files.internal("player.png"));
        playerSprite = new Sprite(playerTexture);
        playerIdleAnimation = new Animation<TextureRegion>(0.500f, new TextureAtlas("player-idle.atlas").getRegions(), Animation.PlayMode.LOOP_PINGPONG);
        enemyIdleAnimation = new Animation<TextureRegion>(0.500f, new TextureAtlas("enemy-idle.atlas").getRegions(), Animation.PlayMode.LOOP_PINGPONG);

        arrowSpritesheet = new ArrowSpritesheet();
        cursorTexture = new Texture("cursor.png");

        stage = new Stage();
        Gdx.input.setInputProcessor(stage);

        table = new Table();
        table.setFillParent(true);
        stage.addActor(table);

        TextButton.TextButtonStyle style = new TextButton.TextButtonStyle();
        //style.up = new TextureRegionDrawable(upRegion);
        //style.down = new TextureRegionDrawable(downRegion);
        style.font = game.font;

        unitNameButton = new TextButton("<unit name>", style);
        table.top().left().add(unitNameButton);

        stageTexture = new Texture("stage.png");

        startNewTeamTurn(Unit.Team.Player);
    }

    @Override
    public void show() {

    }

    void getNextUnit() {
        currentUnit = null;

        for(int i = currentUnitIndex; i < units.size(); i++) {
            if(units.get(i).team == currentTurnTeam) {
                currentUnit = units.get(i);
                currentUnitIndex = i;
                unitNameButton.setText(currentUnit.name);
                return;
            }
        }
    }

    void startNewTeamTurn(Unit.Team team) {
        currentTurnTeam = team;

        currentUnitIndex = 0;
        getNextUnit();

        if(currentUnit != null) {
            smoothlyTransitionCamera(new Vector3(currentUnit.positionX * 16, currentUnit.positionY * 16, 0));

            // reset cursor position
            if(team == Unit.Team.Player) {
                cursorX = currentUnit.positionX;
                cursorY = currentUnit.positionY;
                currentUnit = null;
            }
        }
    }

    void updateUnits() {
        units.removeIf(u -> u.health <= 0);

        // then check if there's a win or lose condition
        long numberOfPlayers = units.stream().filter(u -> u.team == Unit.Team.Player).count();
        long numberOfEnemies = units.stream().filter(u -> u.team == Unit.Team.Enemy).count();

        if(numberOfPlayers == 0)
            gameOver();

        if(numberOfEnemies == 0)
            gameWin();
    }

    boolean hasAllUnitsTakenAction() {
        long numberOfPlayers = units.stream().filter(u -> u.team == currentTurnTeam).count();
        long numberOfUnitsTired = units.stream().filter(u -> u.hasTakenAction).count();

        return numberOfPlayers == numberOfUnitsTired;
    }

    void gameOver() {
        game.setScreen(new GameOverMenuScreen(game));
        dispose();
    }

    void gameWin() {
        // right now let's just return to the main menu
        game.setScreen(new MainMenuScreen(game));
        dispose();
    }

    void advanceTurn() {
        updateUnits();

        if(currentTurnTeam == Unit.Team.Player) {
            if(hasAllUnitsTakenAction()) {
                startNewTeamTurn(Unit.Team.Enemy);
            }
        } else {
            currentUnitIndex++;
            getNextUnit();

            if(currentUnit == null) {
                // reset everything
                currentTurn++;

                for(Unit unit : units) {
                    unit.hasTakenAction = false;
                }

                startNewTeamTurn(Unit.Team.Player);
            } else {
                smoothlyTransitionCamera(new Vector3(currentUnit.positionX * 16, currentUnit.positionY * 16, 0));
            }
        }
    }

    void moveUnitTo(int x, int y) {
        ArrayList<Vector2> path = AStar.path(new Vector2(currentUnit.positionX, currentUnit.positionY), new Vector2((float) x, (float) y), getCollision(), tiledMap);

        // don't allow invalid movements
        if(path != null) {
            Collections.reverse(path);

            currentMovementList = path.subList(0, Math.min(currentUnit.maxDistance, path.size()));
            currentlyTakingAction = true;
        }
    }

    boolean isAdjacent(Unit a, Unit b) {
        ArrayList<Vector2> adjacentPositions = new ArrayList<>();
        adjacentPositions.add(new Vector2(0, -1));
        adjacentPositions.add(new Vector2(0, 1));
        adjacentPositions.add(new Vector2(-1, 0));
        adjacentPositions.add(new Vector2(1, 0));
        adjacentPositions.add(new Vector2(-1, -1));
        adjacentPositions.add(new Vector2(-1, 1));
        adjacentPositions.add(new Vector2(1, -1));
        adjacentPositions.add(new Vector2(1, 1));
        for (Vector2 position : adjacentPositions) {
            int newX = a.positionX + (int)position.x;
            int newY = a.positionY + (int)position.y;

            if(b.positionX == newX && b.positionY == newY)
                return true;
        }

        return false;
    }

    List<UnitAction> getAvailableActionsForPosition(int x, int y) {
        ArrayList<UnitAction> actions = new ArrayList<>();

        for(Unit unit : units) {
            if(unit.positionX == x && unit.positionY == y) {
                if(isAdjacent(currentUnit, unit)) {
                    if (unit.team != currentUnit.team) {
                        actions.add(new AttackAction(currentUnit, unit));
                    } else {
                        // you can't trade with yourself!
                        if (unit != currentUnit) {
                            actions.add(new TradeAction());
                        }
                    }
                }
            }
        }

        actions.add(new WaitAction());

        return actions;
    }

    List<Vector2> getCollision() {
        ArrayList<Vector2> collisions = new ArrayList<>();
        for(Unit unit : units) {
            if(unit != currentUnit) {
                collisions.add(new Vector2(unit.positionX, unit.positionY));
            }
        }

        return collisions;
    }

    void executeAction(UnitAction action) {
        if(action.execute()) {
            advanceTurn();
        }
    }

    Vector3 oldCameraPosition;
    Vector3 targetCameraPosition;
    float lerp = 0.0f;
    boolean currentlyTransitionCamera = false;

    void smoothlyTransitionCamera(Vector3 to) {
        oldCameraPosition = camera.position;
        targetCameraPosition = to;
        lerp = 0.0f;
        currentlyTransitionCamera = true;
    }

    private float stateTime = 0.0f;
    enum Direction {
        Top,
        Bottom,
        Left,
        Right,
        TopLeft,
        TopRight,
        BottomLeft,
        BottomRight
    };

    Direction getFromDelta(Vector2 d) {
        if (d.epsilonEquals(new Vector2(0, -1))) {
            return Direction.Top;
        }

        if (d.epsilonEquals(new Vector2(0, 1))) {
            return Direction.Bottom;
        }

        if (d.epsilonEquals(new Vector2(-1, 0))) {
            return Direction.Right;
        }

        if (d.epsilonEquals(new Vector2(1, 0))) {
            return Direction.Left;
        }

        if (d.epsilonEquals(new Vector2(1, 1))) {
            return Direction.BottomLeft;
        }

        if (d.epsilonEquals(new Vector2(-1, -1))) {
            return Direction.TopRight;
        }

        if (d.epsilonEquals(new Vector2(1, -1))) {
            return Direction.TopLeft;
        }

        if (d.epsilonEquals(new Vector2(-1, 1))) {
            return Direction.BottomRight;
        }

        throw new RuntimeException("Failed to find direction!");
    }

    @Override
    public void render(float delta) {
        ScreenUtils.clear(0, 0, 0.2f, 1);
        stateTime += delta;

        if(currentlyTransitionCamera) {
            camera.position.set(oldCameraPosition.lerp(targetCameraPosition, lerp));
            lerp += delta;

            if(camera.position.epsilonEquals(targetCameraPosition)) {
                currentlyTransitionCamera = false;
            }
        }

        camera.update();
        tiledMapRenderer.setView(camera);
        tiledMapRenderer.render();

        stage.act(Gdx.graphics.getDeltaTime());
        stage.draw();

        game.batch.begin();
        game.batch.setProjectionMatrix(camera.combined);

        for(Unit unit : units) {

            TextureRegion currentFrame = null;
            if(unit.team == Unit.Team.Player) {
                currentFrame = playerIdleAnimation.getKeyFrame(stateTime, true);
            } else {
                currentFrame = enemyIdleAnimation.getKeyFrame(stateTime, true);
            }

            if(unit.hasTakenAction) {
                game.batch.setColor(0.5f, 0.5f, 0.5f, 1.0f);
            } else {
                game.batch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            }

            game.batch.draw(currentFrame, unit.positionX * 16, unit.positionY * 16);
        }

        // early exit, since we want the camera to finish its nice transition
        if(currentlyTransitionCamera) {
            game.batch.end();
            return;
        }

        if(currentlyParticipatingInStageEvent) {
            game.batch.end();

            Gdx.gl.glEnable(GL20.GL_BLEND);
            Gdx.gl.glBlendFunc(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);

            ShapeRenderer renderer = new ShapeRenderer();
            renderer.begin(ShapeRenderer.ShapeType.Filled);
            renderer.set(ShapeRenderer.ShapeType.Filled);
            renderer.setProjectionMatrix(camera.combined);
            renderer.rect(
                    camera.position.x - camera.viewportWidth,
                    camera.position.y - camera.viewportHeight,
                    camera.viewportWidth * 2,
                    camera.viewportHeight * 2,
                    new Color(0.0f, 0.0f, 0.0f, 0.5f),
                    new Color(0.0f, 0.0f, 0.0f, 0.5f),
                    new Color(0.0f, 0.0f, 0.0f, 0.5f),
                    new Color(0.0f, 0.0f, 0.0f, 0.5f)
            );
            renderer.end();

            game.batch.begin();

            game.batch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            game.batch.draw(stageTexture, camera.position.x - stageTexture.getWidth() / 2, camera.position.y - stageTexture.getHeight() / 2);

            game.batch.end();

            return;
        }

        if(currentTurnTeam == Unit.Team.Player) {
            if(currentUnit == null) {
                if (Gdx.input.isKeyPressed(Input.Keys.ENTER) || Gdx.input.isButtonPressed(0)) {
                    for (Unit unit : units) {
                        if (unit.team == Unit.Team.Player && !unit.hasTakenAction) {
                            if (cursorX == unit.positionX && cursorY == unit.positionY) {
                                currentUnit = unit;
                            }
                        }
                    }
                }
            } else {
                // player is trying to move
                if (Gdx.input.isButtonPressed(0) && !currentlyAwaitingAuxilaryAction) {
                    hasStartedMoving = true;

                    Vector3 mousePos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0);

                    camera.unproject(mousePos);

                    double newX = Math.floor(mousePos.x);
                    double newY = Math.floor(mousePos.y);

                    int newFixedX = (int) (newX / 16.0);
                    int newFixedY = (int) (newY / 16.0);

                    ArrayList<Vector2> path = AStar.path(new Vector2(currentUnit.positionX, currentUnit.positionY), new Vector2((float) newFixedX, (float) newFixedY), getCollision(), tiledMap);
                    if (path != null && path.size() > 1) {
                        Collections.reverse(path);
                        List<Vector2> realPath = path.subList(0, Math.min(currentUnit.maxDistance, path.size()));

                        // we start at index 1 since there's no point in drawing an arrow over the player
                        for (int i = 1; i < realPath.size() - 1; i++) {
                            Vector2 previousPosition = realPath.get(i - 1);
                            Vector2 newPosition = realPath.get(i);
                            Vector2 nextPosition = realPath.get(i + 1);

                            // we want to grab two directions, first is our "entrance" and then our "exit". of course arrows that are just a position don't exist
                            Direction entrance = getFromDelta(new Vector2(newPosition.x - previousPosition.x, newPosition.y - previousPosition.y));
                            Direction exit = getFromDelta(new Vector2(newPosition.x - nextPosition.x, newPosition.y - nextPosition.y));

                            TextureRegion spriteRegion = null;
                            TextureRegion leftSprite = null;
                            TextureRegion rightSprite = null;

                            // TODO: condense duplicate branches

                            // handle straights
                            if(entrance == Direction.Bottom && exit == Direction.Top) {
                                spriteRegion = arrowSpritesheet.getUpDown();
                            }

                            if(entrance == Direction.Top && exit == Direction.Bottom) {
                                spriteRegion = arrowSpritesheet.getUpDown();
                            }

                            if(entrance == Direction.Left && exit == Direction.Right) {
                                spriteRegion = arrowSpritesheet.getLeftRight();
                            }

                            if(entrance == Direction.Right && exit == Direction.Left) {
                                spriteRegion = arrowSpritesheet.getLeftRight();
                            }

                            // handle crosses
                            if(entrance == Direction.TopLeft && exit == Direction.BottomRight) {
                                spriteRegion = arrowSpritesheet.getDiagDown();
                            }

                            if(entrance == Direction.TopLeft && exit == Direction.TopLeft) {
                                spriteRegion = arrowSpritesheet.getDiagDown();
                            }

                            if(entrance == Direction.BottomLeft && exit == Direction.BottomLeft) {
                                spriteRegion = arrowSpritesheet.getDiagUp();
                            }

                            if(entrance == Direction.BottomLeft && exit == Direction.TopRight) {
                                spriteRegion = arrowSpritesheet.getDiagUp();
                            }

                            if(entrance == Direction.BottomRight && exit == Direction.TopLeft) {
                                spriteRegion = arrowSpritesheet.getDiagDown();
                            }

                            if(entrance == Direction.TopRight && exit == Direction.BottomLeft) {
                                spriteRegion = arrowSpritesheet.getDiagDown();
                            }

                            if(spriteRegion == arrowSpritesheet.getDiagDown()) {
                                leftSprite = arrowSpritesheet.getCornerTopRight();
                                rightSprite = arrowSpritesheet.getCornerBottomLeft();
                            }

                            if(spriteRegion == arrowSpritesheet.getDiagUp()) {
                                leftSprite = arrowSpritesheet.getCornerBottomRight();
                                rightSprite = arrowSpritesheet.getCornerTopLeft();
                            }

                            // handle bends
                            if(entrance == Direction.BottomRight && exit == Direction.Top) {
                                spriteRegion = arrowSpritesheet.getBendBottomRightToTop();
                            }

                            if(entrance == Direction.Top && exit == Direction.BottomRight) {
                                spriteRegion = arrowSpritesheet.getBendBottomRightToTop();
                            }

                            if(entrance == Direction.BottomLeft && exit == Direction.Top) {
                                spriteRegion = arrowSpritesheet.getBendBottomLeftToTop();
                            }

                            if(entrance == Direction.Top && exit == Direction.BottomLeft) {
                                spriteRegion = arrowSpritesheet.getBendBottomLeftToTop();
                            }

                            if(entrance == Direction.TopLeft && exit == Direction.Bottom) {
                                spriteRegion = arrowSpritesheet.getBendTopLeftToBottom();
                            }

                            if(entrance == Direction.Bottom && exit == Direction.TopLeft) {
                                spriteRegion = arrowSpritesheet.getBendTopLeftToBottom();
                            }

                            if(entrance == Direction.TopRight && exit == Direction.Bottom) {
                                spriteRegion = arrowSpritesheet.getBendTopRightToBottom();
                            }

                            if(entrance == Direction.Bottom && exit == Direction.TopRight) {
                                spriteRegion = arrowSpritesheet.getBendTopRightToBottom();
                            }

                            if(entrance == Direction.TopLeft && exit == Direction.Right) {
                                spriteRegion = arrowSpritesheet.getBendTopLeftToRight();
                            }

                            if(entrance == Direction.Right && exit == Direction.TopLeft) {
                                spriteRegion = arrowSpritesheet.getBendTopLeftToRight();
                            }

                            if(entrance == Direction.BottomLeft && exit == Direction.Right) {
                                spriteRegion = arrowSpritesheet.getBendBottomLeftToRight();
                            }

                            if(entrance == Direction.Right && exit == Direction.BottomLeft) {
                                spriteRegion = arrowSpritesheet.getBendBottomLeftToRight();
                            }

                            //game.font.draw(game.batch, entrance.toString() + " to " + exit.toString(), newPosition.x * 16, newPosition.y * 16);

                            if (spriteRegion != null) {
                                game.batch.draw(spriteRegion, newPosition.x * 16, newPosition.y * 16);
                            }

                            if(leftSprite != null) {
                                game.batch.draw(leftSprite, (newPosition.x - 1) * 16, newPosition.y * 16);
                            }

                            if(rightSprite != null) {
                                game.batch.draw(rightSprite, (newPosition.x + 1) * 16, newPosition.y * 16);
                            }
                        }

                        // now we want to draw the arrow or the "end" point.
                        Vector2 secondToLastPosition = realPath.get(realPath.size() - 2);
                        Vector2 lastPosition = realPath.get(realPath.size() - 1);

                        Direction exit = getFromDelta(new Vector2(lastPosition.x - secondToLastPosition.x, lastPosition.y - secondToLastPosition.y));

                        TextureRegion spriteRegion = null;

                        switch(exit) {
                            case Top:
                                spriteRegion = arrowSpritesheet.getArrowFromTop();
                                break;
                            case Bottom:
                                spriteRegion = arrowSpritesheet.getArrowFromBottom();
                                break;
                            case Left:
                                spriteRegion = arrowSpritesheet.getArrowFromLeft();
                                break;
                            case Right:
                                spriteRegion = arrowSpritesheet.getArrowFromRight();
                                break;
                            case TopLeft:
                                spriteRegion = arrowSpritesheet.getArrowFromTopLeft();
                                break;
                            case TopRight:
                                spriteRegion = arrowSpritesheet.getArrowFromTopRight();
                                break;
                            case BottomLeft:
                                spriteRegion = arrowSpritesheet.getArrowFromBottomLeft();
                                break;
                            case BottomRight:
                                spriteRegion = arrowSpritesheet.getArrowFromBottomRight();
                                break;
                        }

                        if(spriteRegion != null) {
                            game.batch.draw(spriteRegion, lastPosition.x * 16, lastPosition.y * 16);
                        }
                    }
                } else if (hasStartedMoving) {
                    Vector3 mousePos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0);

                    camera.unproject(mousePos);

                    double newX = Math.floor(mousePos.x);
                    double newY = Math.floor(mousePos.y);

                    moveUnitTo((int) (newX / 16.0), (int) (newY / 16.0));

                    hasStartedMoving = false;
                }
            }
        }

        if(currentTurnTeam == Unit.Team.Enemy && currentUnit != null && !currentlyTakingAction && !currentlyAwaitingAuxilaryAction) {
            // first we want to choose the closest enemy unit.
            Unit closestUnit = null;
            float closestDistance = 999;
            for(Unit unit : units) {
                if(unit.team == Unit.Team.Player) {
                    float dist = Vector2.dst(unit.positionX, unit.positionY, currentUnit.positionX, currentUnit.positionY);
                    if(dist < closestDistance) {
                        closestUnit = unit;
                        closestDistance = dist;
                    }
                }
            }

            if(closestUnit != null) {
                // if we're already next to them, why move?
                if (isAdjacent(currentUnit, closestUnit)) {
                    currentlyAwaitingAuxilaryAction = true;
                } else {
                    // now that we have the closest unit, lets path to it.
                    ArrayList<Vector2> path = AStar.path(new Vector2(currentUnit.positionX, currentUnit.positionY), new Vector2((float) closestUnit.positionX, (float) closestUnit.positionY), getCollision(), tiledMap);
                    if (path != null) {
                        Collections.reverse(path);
                        List<Vector2> realPath = path.subList(0, Math.min(currentUnit.maxDistance, path.size()));

                        // we want to grab the second to last position, since that's the "max distance" we can go, and it makes no sense
                        // for units to go shorter
                        Vector2 determinedPos = realPath.get(realPath.size() - 2);
                        moveUnitTo((int) determinedPos.x, (int) determinedPos.y);
                    }
                }
            } else {
                // TODO: this should be normal behavior if a player unit is not in "range" of an enemy unit
                advanceTurn();
            }
        }

        if(currentlyTakingAction) {
            if(!currentMovementList.isEmpty()) {
                Vector2 nextMove = currentMovementList.remove(0);
                currentUnit.positionX = (int) nextMove.x;
                currentUnit.positionY = (int) nextMove.y;

                smoothlyTransitionCamera(new Vector3(currentUnit.positionX * 16, currentUnit.positionY * 16, 0));
            } else {
                currentlyTakingAction = false;
                currentlyAwaitingAuxilaryAction = true;
            }
        }

        if(currentlyAwaitingAuxilaryAction && currentUnit != null) {
            if (currentUnit.team == Unit.Team.Enemy) {
                Unit closestUnit = null;
                float closestDistance = 999;
                for(Unit unit : units) {
                    if(unit.team == Unit.Team.Player) {
                        float dist = Vector2.dst(unit.positionX, unit.positionY, currentUnit.positionX, currentUnit.positionY);
                        if(dist < closestDistance) {
                            closestUnit = unit;
                            closestDistance = dist;
                        }
                    }
                }

                currentlyAwaitingAuxilaryAction = false;

                if(isAdjacent(closestUnit, currentUnit)) {
                    AttackAction action = new AttackAction(currentUnit, closestUnit);
                    currentUnit.hasTakenAction = true;
                    executeAction(action);
                } else {
                    currentUnit.hasTakenAction = true;
                    advanceTurn();
                }
            } else {
                Vector3 mousePos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0);

                camera.unproject(mousePos);

                double newX = Math.floor(mousePos.x);
                double newY = Math.floor(mousePos.y);

                int newFixedX = (int) (newX / 16.0);
                int newFixedY = (int) (newY / 16.0);

                List<UnitAction> actions = getAvailableActionsForPosition(newFixedX, newFixedY);
                for (int i = 0; i < actions.size(); i++) {
                    game.font.draw(game.batch, (i + 1) + ": " + actions.get(i).getName(), (float) newX, (float) newY + (i * 32));

                    if (Gdx.input.isKeyPressed(Input.Keys.NUM_1 + i)) {
                        currentlyAwaitingAuxilaryAction = false;
                        currentUnit.hasTakenAction = true;
                        currentUnit = null;
                        executeAction(actions.get(i));
                    }
                }
            }
        }

        if(currentTurnTeam == Unit.Team.Player && currentUnit == null) {
            Vector3 mousePos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0);

            camera.unproject(mousePos);

            double newX = Math.floor(mousePos.x);
            double newY = Math.floor(mousePos.y);

            int newFixedX = (int) (newX / 16.0);
            int newFixedY = (int) (newY / 16.0);

            cursorX = newFixedX;
            cursorY = newFixedY;

            game.batch.draw(cursorTexture, cursorX * 16, cursorY * 16);
        }

        game.batch.end();
    }

    @Override
    public void resize(int width, int height) {
        camera.viewportWidth = 230;
        camera.viewportHeight = 230 * ((float)height / (float)width);

        stage.getViewport().update(width, height, true);
    }

    @Override
    public void pause() {

    }

    @Override
    public void resume() {

    }

    @Override
    public void hide() {

    }

    @Override
    public void dispose() {
        stage.dispose();
    }
}
